#!/usr/bin/env python3

import tempfile
import os
import re
#import xml.etree.ElementTree as ET
import lxml.etree as ET


import gcodeParser
from processor3D import amf



class AMF():
    def __init__(self, file):
        self.amf_file = file
        self.open()
        self.modificators = {}
        self.slice_config = {}
        self.bbox = gcodeParser.BBox()


    def __repr__(self):
        return "<{} instance at {}, file: {}>".format(self.__class__.__name__, id(self), self.amf_file)

    def __str__(self):
        return """
AMF file {}:
\t vertex {}
\t faces {}
""".format(self.amf_file, 0, 0)

    def open(self):
        self.amf = ET.parse(self.amf_file)

    def write(self, new_name = None, build = True):
        print("Ukladam AMF soubor")
        if build:
            self.build()
        if new_name:
            filename = new_name
        else:
            filename = self.amf_file
        self.amf.write(filename, encoding='UTF-8', pretty_print=True)

        return filename

    def set_amf(self, file):
        self.amf = file

    def get_file(self):
        return os.path.abspath(self.amf_file)

    def get_amf(self):
        pass

    def add_modificator(self, name, AMF):
        self.modificators[name] = AMF

    def modificator(self, name = None):
        if name == 'main' or name == None:
            return self
        return self.modificators[name]

    def get_volume(self, index = 0):
        volume = self.amf.find('object').find('mesh').find('volume')
        return volume

    def get_vertices(self, index = 0):
        vertices = self.amf.find('object').find('mesh').find('vertices')
        return vertices

    def set_slicing_config(self, file):
        file = os.path.abspath(os.path.join(os.path.dirname(self.amf_file), file))
        print("Config:", file)
        root = self.amf.findall('object')[0].find('mesh').find('volume')
        f = open(file)
        for row in f:
            if row.startswith('#'):
                continue
            key, value = row.strip().split('=')
            if value.strip() == '':
                continue
            self.slice_config[key.strip()] = value.strip()
        return file

    def set_slicing_parameter(self, parameter, value):
        root = self.amf.findall('object')[0].find('mesh').find('volume')
        metadata = ET.SubElement(root, 'metadata')
        metadata.set('type', 'slic3r.{}'.format(parameter))
        metadata.text = value
        metadata.tail = '\n'

    def add_new_modificator(self, amf_modificator):
        pass

    def get_BBox(self):
        for vertex in self.get_vertices().iter():
            if vertex.tag == 'coordinates':
                coord = {}
                for child in vertex:
                    coord[child.tag] = child.text

                coord['X'] = float(coord['x'])
                coord['Y'] = float(coord['y'])
                coord['Z'] = float(coord['z'])
                self.bbox.extend(coord)
        return self.bbox

    def build_slicing_parameters(self, placeholder = None):
        if not placeholder:
            placeholder = self.amf.find('object').find('mesh').find('volume')
        for key in self.slice_config:
            value = self.slice_config[key]
            #print("CFG", key, value, placeholder)
            metadata = ET.SubElement(placeholder, 'metadata')
            metadata.set('type', 'slic3r.{}'.format(key))
            metadata.text = value
            metadata.tail = '\n'

    def build(self):
        #self.build_slicing_parameters(self.amf.find('object').find('mesh').find('volume'))
        self.build_slicing_parameters(self.amf.find('object'))
        #self.get_BBox()

        for modificator_name in self.modificators:
            print("APLIKUJI MODIFIKATOR", modificator_name)
            modificator = self.modificators[modificator_name]
            modificator.build_slicing_parameters()
            volume = modificator.get_volume()
            vertices = modificator.get_vertices()
            vertices_count = len(self.get_vertices())
            modificator_volume = ET.SubElement(self.amf.find('object').find('mesh'), 'volume')

            for x in volume.iterfind('triangle'):
                #print(list(x)[0].text, str(int(list(x)[0].text)+vertices_count))
                triangle = ET.SubElement(modificator_volume, 'triangle')
                v1l = ET.SubElement(triangle, 'v1')
                v1l.text = str(int(list(x)[0].text)+vertices_count)
                v2l = ET.SubElement(triangle, 'v2')
                v2l.text = str(int(list(x)[1].text)+vertices_count)
                v3l= ET.SubElement(triangle, 'v3')
                v3l.text = str(int(list(x)[2].text)+vertices_count)
                triangle.tail = '\n'

            param = {'name': modificator_name, 'slic3r.modifier':1}
            for x in param:
                metadata = ET.SubElement(modificator_volume, 'metadata')
                metadata.set('type', x)
                metadata.text = str(param[x])
                metadata.tail = '\n'

            modificator.build_slicing_parameters(modificator_volume)

            for x in vertices.iterfind('vertex'):
                self.amf.find('object').find('mesh').find('vertices').append(x)
