#!/usr/bin/env python3

import re
import subprocess
import tempfile
import os
import re
from termcolor import colored, cprint

import gcodeParser
from processor3D import amf



class Extensions(list):
    def __init__(self, *args, **kwargs):
        super(Extensions, self).__init__()

    def add(self, extension):
        self.append(extension)

class Extension(dict):
    method = None
    parameters = []

    def __init__(self, row):
        super(Extension, self).__init__()
        param_start = row.find('(')
        param_stop = row.find(')')
        arg = re.search(r'\((.*?)\)', row).group(0)

        self['command'] = row[0:param_start]
        self['arguments'] = row[param_start+1:param_stop].split(',')
        if self['arguments'] == ['']: self['arguments'] = None


class EScadParser(object):
    """docstring for EScadParser."""

    def __init__(self, arg = None):
        super(EScadParser, self).__init__()
        self.arg = arg

        self.extensions = Extensions()

    def parse_file(self, file):
        self.escad_file = file
        self.load_file(file)

    def load_file(self, filename):
        f = open(filename, "r")
        for row in f:
            row = row.replace(" ", "")
            #print(row)

            if '//@' in row:
                self.extensions.add(Extension(row[3:]))

    def get_extensions(self):
        return self.extensions



class ScadRender(object):

    def __init__(self, file):
        super(ScadRender, self).__init__()
        self.scad_file = file
        self.rendered = False
        # TODO: OpenSCAD from configuration
        self.scad_binary = "openscad-nightly"
        self.scad_binary = "/home/roman/programs/OpenSCAD-2019.05.19.ai2702-x86_64.AppImage"
        #self.scad_binary = os.path.abspath(os.path.join(os.path.dirname(__file__), '../resources/OpenSCAD-2019.12.21.AppImage'))

    def render_file(self, out_file, type=None):
        options = [self.scad_binary]
        options += ['-o', out_file]
        options += [self.scad_file]

        cprint(' '.join(options), 'yellow')
        subprocess.call(' '.join(options), shell=True)
        self.rendered = out_file
        return self.get_mesh_file()

    def export_image(self, out_file, type = None):
        amf_filename = self.amf_file.get_file()

        file = '''
            import({});
        '''.format(amf_filename)
        filename = '{}/{}.mscad'.format(tempfile.gettempdir(), os.path.basename(out_file))
        f = open(filename, 'w')
        f.write(file)
        f.close()

        options = [self.scad_binary]
        options += ['-o', out_file]
        options += [self.scad_file]
        options += ['--viewall']
        options += ['--autocenter']

        cprint(' '.join(options), 'yellow')
        subprocess.call(' '.join(options), shell=True)
        return out_file

    def export_stl(self, out_file, type=None):
        amf_filename = self.amf_file.get_file()

        file = '''
            import({});
        '''.format(amf_filename)
        filename = '{}/{}.mscad'.format(tempfile.gettempdir(), os.path.basename(out_file))
        f = open(filename, 'w')
        f.write(file)
        f.close()

        options = [self.scad_binary]
        options += ['-o', out_file]
        options += [self.scad_file]
        options += ['--viewall']
        options += ['--autocenter']

        cprint(' '.join(options), 'yellow')
        subprocess.call(' '.join(options), shell=True)
        return out_file

    def render_module(self, out_file, module, type=None):
        out_file = out_file.replace('.amf', 'modif_'+module+'.amf')
        out_file = out_file.replace('.scad', 'modif_'+module+'.amf')
        file = '''
use <{}>
{}();
        '''.format(os.path.abspath(self.scad_file), module)
        filename = '{}/{}.mscad'.format(tempfile.gettempdir(), module)
        f = open(filename, 'w')
        f.write(file)
        f.close()

        options = [self.scad_binary]
        options += ['-o', out_file]
        options += [filename]

        cprint(' '.join(options), 'yellow')
        subprocess.call(' '.join(options), shell=True)
        self.rendered = out_file
        return self.get_mesh_file()

    def get_mesh_file(self):
        self.amf_file = amf.AMF(self.rendered)
        return self.amf_file
